/*************************************************************/
/**                                                         **/
/**                        MakeFTab.c                       **/
/**                                                         **/
/** This program is used to create the note and pitch wheel **/
/** tables for use with PSG2Mid                             **/
/**                                                         **/
/** Copyright (C) Marcel de Kogel 1996                      **/
/**     You are not allowed to distribute this software     **/
/**     commercially. Please, notify me, if you make any    **/
/**     changes to this file.                               **/
/**                                                         **/
/*************************************************************/

#include <stdio.h>
#include <math.h>

static double getnote (double freq)
{
 return 12*log(freq*32/261.63)/log(2);
}

void main (void)
{
 FILE *outfile;
 int i;
 double freq;
 double note;
 int midinote;
 int midiwheel;
 outfile=fopen ("ftable.h","wb");
 if (!outfile)
 {
  printf ("Can't open ftable.h\n");
  return;
 }
 for (i=0;i<4096;++i)
 {
  if (i)
   freq=3579545.0/(32*i);
  else
   freq=10000000;
  note=getnote (freq);
  if (note>127)
   note=127;
  midinote=floor(note+0.5);
  midiwheel=8192+((note-midinote)*8192/24);
  fprintf (outfile,"{ %3d,%5d }%s\n",midinote,midiwheel,",");
 }
 fprintf (outfile,"\n/* OPLL data follows */\n\n");
 for (i=0;i<4096;++i)
 {
  int block,f;
  f=(i&0x1FF)<<1; if (!f) f=1;
  block=(i>>9)&7;
  freq=50000*f*pow(2,block-20);
  note=getnote (freq);
  if (note>127)
   note=127;
  if (note<0)
   note=0;
  midinote=floor(note+0.5);
  midiwheel=8192+((note-midinote)*8192/24);
  fprintf (outfile,"{ %3d,%5d }%s\n",midinote,midiwheel,(i==4095)? "":",");
 }
 fclose (outfile);
}

